# overrides https://github.com/dbt-labs/dbt-bigquery/blob/main/dbt/adapters/bigquery/connections.py#L442
def execute(self, sql, auto_begin=False, fetch=None):
    from alvin_integration.producers.dbt.lineage.extractors.bigquery import (
        AlvinBigQueryAdapterResponse,
    )
    from dbt.clients import agate_helper

    from datetime import datetime, timezone
    event_time = datetime.now(timezone.utc)

    sql = self._add_query_comment(sql)
    # auto_begin is ignored on bigquery, and only included for consistency
    query_job, iterator = self.raw_execute(sql, fetch=fetch)

    if fetch:
        table = self.get_table_from_response(iterator)
    else:
        table = agate_helper.empty_table()

    message = "OK"
    code = None
    num_rows = None
    bytes_processed = None
    location = None
    job_id = None
    project_id = None
    num_rows_formatted = None
    processed_bytes = None
    slot_ms = None

    if query_job.statement_type == "CREATE_VIEW":
        code = "CREATE VIEW"

    elif query_job.statement_type == "CREATE_TABLE_AS_SELECT":
        code = "CREATE TABLE"
        conn = self.get_thread_connection()
        client = conn.handle
        query_table = client.get_table(query_job.destination)
        num_rows = query_table.num_rows

    elif query_job.statement_type == "SCRIPT":
        code = "SCRIPT"

    elif query_job.statement_type in ["INSERT", "DELETE", "MERGE", "UPDATE"]:
        code = query_job.statement_type
        num_rows = query_job.num_dml_affected_rows

    elif query_job.statement_type == "SELECT":
        code = "SELECT"
        conn = self.get_thread_connection()
        client = conn.handle
        # use anonymous table for num_rows
        query_table = client.get_table(query_job.destination)
        num_rows = query_table.num_rows

    # set common attributes
    bytes_processed = query_job.total_bytes_processed
    slot_ms = query_job.slot_millis
    processed_bytes = self.format_bytes(bytes_processed)
    location = query_job.location
    job_id = query_job.job_id
    project_id = query_job.project
    if num_rows is not None:
        num_rows_formatted = self.format_rows_number(num_rows)
        message = f"{code} ({num_rows_formatted} rows, {processed_bytes} processed)"
    elif bytes_processed is not None:
        message = f"{code} ({processed_bytes} processed)"
    else:
        message = f"{code}"

    if location is not None and job_id is not None and project_id is not None:
        from dbt.events import AdapterLogger
        logger = AdapterLogger("BigQuery")
        logger.debug(self._bq_job_link(location, project_id, job_id))

    response = AlvinBigQueryAdapterResponse(
        event_time=event_time,
        _message=message,
        rows_affected=num_rows,
        code=code,
        bytes_processed=bytes_processed,
        started=query_job.started,
        ended=query_job.ended,
        location=location,
        project_id=project_id,
        job_id=job_id,
        slot_ms=slot_ms,
    )

    return response, table



import logging
import os

import requests

from alvin_integration.helper import AlvinLoggerAdapter, log_verbose


class AlvinLineageBackendAdapter:
    def __init__(self, alvin_base_url=None, alvin_api_key=None):
        self.alvin_base_url = os.environ.get("ALVIN_URL", alvin_base_url)

        self.alvin_api_key = os.environ.get("ALVIN_API_KEY", alvin_api_key)

        self.log = AlvinLoggerAdapter(logging.getLogger(__name__), {})

    def send_data(self, payload, resource):
        try:
            requests.post(
                f"{self.alvin_base_url}/{resource}",
                json=payload,
                headers={"X-API-KEY": self.alvin_api_key},
            )
        except Exception as err:
            log_verbose(f"Error sending message to Alvin backend. Error: {err}.")



from dataclasses import dataclass
from datetime import datetime

from dbt.adapters.bigquery.connections import BigQueryAdapterResponse


@dataclass
class AlvinBigQueryAdapterResponse(BigQueryAdapterResponse):
    job_id: str = None
    alvin_platform_id: str = None
    event_time: datetime = None
    started: datetime = None
    ended: datetime = None



import os
from dataclasses import asdict
from importlib.metadata import version
from typing import Optional

from alvin_integration.helper import log_verbose
from openlineage.airflow.extractors.base import TaskMetadata

from alvin_integration.constants import ALVIN_PACKAGE_NAME
from alvin_integration.models import AlvinAirflowTaskExecution


class AlvinAirflowExtractorMixin:
    def build_facet(self, task_instance):
        raise NotImplementedError

    def build_task_metadata(self, task_instance):
        raise NotImplementedError

    def get_airflow_run_id(self, task_instance):
        raise NotImplementedError

    def get_connection_id(self):
        raise NotImplementedError

    def get_alvin_package_version(self):
        return version(ALVIN_PACKAGE_NAME)

    def get_alvin_platform_id(self):
        return os.getenv("ALVIN_PLATFORM_ID")

    def extract_on_complete(self, task_instance) -> Optional[TaskMetadata]:

        if not task_instance:
            return self.extract()

        alvin_facet = self.build_facet(task_instance)

        task_metadata = self.extract()

        if not task_metadata:
            task_metadata = self.build_task_metadata(task_instance)

        log_verbose(f"Alvin Facet: {asdict(alvin_facet)}")

        task_metadata.job_facets.update(asdict(alvin_facet))

        return task_metadata

    def get_execution_details(self, task_instance):
        return AlvinAirflowTaskExecution(
            task_id=task_instance.task_id,
            dag_id=task_instance.dag_id,
            run_id=self.get_airflow_run_id(task_instance),
            start_date=task_instance.start_date.isoformat(),
            end_date=task_instance.end_date.isoformat()
            if task_instance.end_date
            else None,
            duration=task_instance.duration,
            try_number=task_instance.try_number,
            pool=task_instance.pool,
            operator=task_instance.operator,
            state=task_instance.state,
            max_tries=task_instance.max_tries,
            platform_id=self.get_alvin_platform_id(),
            alvin_package_version=self.get_alvin_package_version(),
        )



from abc import ABC, abstractmethod
from typing import List

from alvin_integration.models import AlvinPatch


class AbstractProducerConfig(ABC):
    @property
    @abstractmethod
    def producer_name(self):
        pass

    @abstractmethod
    def get_patching_list(self) -> List[AlvinPatch]:
        pass

    @abstractmethod
    def get_target_packages(self):
        pass

    @abstractmethod
    def get_target_pipelines(self):
        pass

    @abstractmethod
    def get_lineage_config(self):
        pass
